<?php
/*
Plugin Name: 커스텀 목차 플러그인
Description: 게시글 상단에 자동으로 목차를 추가하는 플러그인입니다.
Version: 1.3
Author: kongdy
*/


// 관리 페이지 추가
function toc_admin_menu() {
    add_menu_page(
        '콩디 목차 설정',
        '콩디 목차',
        'manage_options',
        'toc-settings',
        'toc_settings_page',
        'dashicons-editor-ul',
        100
    );
}
add_action('admin_menu', 'toc_admin_menu');

// 설정 페이지 렌더링
function toc_settings_page() {
    ?>
    <div class="wrap">
     <a href="https://xn--hy1b53d22msjebow.com/%EB%AC%B8%EC%9D%98-%EA%B2%8C%EC%8B%9C%ED%8C%90" class="button button-primary kongdy-contact-button" target="_blank">문의하기</a>
        <h1>콩디 목차 설정</h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('toc_settings_group');
            do_settings_sections('toc-settings');
            submit_button();
            ?>
        </form>
    </div>
    <?php
}

// 설정 필드 등록
function toc_settings_init() {
    register_setting('toc_settings_group', 'toc_position');
    register_setting('toc_settings_group', 'toc_h2_symbol');
    register_setting('toc_settings_group', 'toc_h3_symbol');
    register_setting('toc_settings_group', 'toc_h4_symbol');
    register_setting('toc_settings_group', 'toc_h2_font_size');
    register_setting('toc_settings_group', 'toc_h3_font_size');
    register_setting('toc_settings_group', 'toc_h4_font_size');
    register_setting('toc_settings_group', 'toc_table_background_color');
    register_setting('toc_settings_group', 'toc_hover_background_color');
    register_setting('toc_settings_group', 'toc_table_width');

    add_settings_section(
        'toc_settings_section',
        '목차 기본 설정',
        'toc_settings_section_callback',
        'toc-settings'
    );

    add_settings_field(
        'toc_position',
        '목차 위치',
        'toc_position_callback',
        'toc-settings',
        'toc_settings_section'
    );

    add_settings_field(
        'toc_table_width',
        '테이블 넓이',
        'toc_table_width_callback',
        'toc-settings',
        'toc_settings_section'
    );

    add_settings_field(
        'toc_h2_settings',
        'H2 설정',
        'toc_h2_settings_callback',
        'toc-settings',
        'toc_settings_section'
    );

    add_settings_field(
        'toc_h3_settings',
        'H3 설정',
        'toc_h3_settings_callback',
        'toc-settings',
        'toc_settings_section'
    );

    add_settings_field(
        'toc_h4_settings',
        'H4 설정',
        'toc_h4_settings_callback',
        'toc-settings',
        'toc_settings_section'
    );

    add_settings_field(
        'toc_table_background_color',
        '테이블 배경 색상',
        'toc_table_background_color_callback',
        'toc-settings',
        'toc_settings_section'
    );

    add_settings_field(
        'toc_hover_background_color',
        '후버 배경 색상',
        'toc_hover_background_color_callback',
        'toc-settings',
        'toc_settings_section'
    );
}
add_action('admin_init', 'toc_settings_init');

// 설정 섹션 설명
function toc_settings_section_callback() {
    echo '<p>목차 설정을 아래에서 변경할 수 있습니다.</p>';
}

// 목차 위치 필드 렌더링
function toc_position_callback() {
    $toc_position = get_option('toc_position', 'after_first_paragraph');
    ?>
    <select name="toc_position">
        <option value="top_of_page" <?php selected($toc_position, 'top_of_page'); ?>>페이지 상단</option>
        <option value="after_first_paragraph" <?php selected($toc_position, 'after_first_paragraph'); ?>>첫 번째 단락 이후</option>
    </select>
    <?php
}

// 테이블 넓이 필드 렌더링
function toc_table_width_callback() {
    $toc_table_width = get_option('toc_table_width', '75%');
    ?>
    <input type="text" name="toc_table_width" id="toc_table_width" value="<?php echo esc_attr($toc_table_width); ?>" readonly style="width: 70px;"/>
    <input type="range" name="toc_table_width_range" id="toc_table_width_range" min="50" max="100" value="<?php echo intval($toc_table_width); ?>" oninput="updateTableWidth(this.value)" />
    <script>
        function updateTableWidth(value) {
            var tableWidth = value + '%';
            document.getElementById('toc_table_width').value = tableWidth;
            document.querySelector('.toc-table2').style.width = tableWidth;
        }

        document.addEventListener('DOMContentLoaded', function () {
            var initialWidth = document.getElementById('toc_table_width').value;
            document.querySelector('.toc-table2').style.width = initialWidth;
            document.getElementById('toc_table_width_range').value = parseInt(initialWidth);
        });
    </script>
    <?php
}

// H2 설정 필드 렌더링
function toc_h2_settings_callback() {
    $toc_h2_symbol = get_option('toc_h2_symbol', '');
    $toc_h2_font_size = get_option('toc_h2_font_size', '22px');
    ?>
    <label>기호:
        <select name="toc_h2_symbol">
            <option value="" <?php selected($toc_h2_symbol, ''); ?>>없음</option>
            <option value="▸" <?php selected($toc_h2_symbol, '▸'); ?>>▸</option>
            <option value="•" <?php selected($toc_h2_symbol, '•'); ?>>•</option>
            <option value="✓" <?php selected($toc_h2_symbol, '✓'); ?>>✓</option>
            <option value="➤" <?php selected($toc_h2_symbol, '➤'); ?>>➤</option>
            <option value="➔" <?php selected($toc_h2_symbol, '➔'); ?>>➔</option>
        </select>
    </label>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
    <label> 글자 크기:
        <input type="range" name="toc_h2_font_size" min="12" max="36" value="<?php echo intval($toc_h2_font_size); ?>" oninput="document.getElementById('h2_font_size_value').innerHTML = this.value + 'px';" />
        <span id="h2_font_size_value"><?php echo esc_attr($toc_h2_font_size); ?></span>
    </label>
    <?php
}

// H3 설정 필드 렌더링
function toc_h3_settings_callback() {
    $toc_h3_symbol = get_option('toc_h3_symbol', '▸');
    $toc_h3_font_size = get_option('toc_h3_font_size', '20px');
    ?>
    <label>기호:
        <select name="toc_h3_symbol">
            <option value="" <?php selected($toc_h3_symbol, ''); ?>>없음</option>
            <option value="▸" <?php selected($toc_h3_symbol, '▸'); ?>>▸</option>
            <option value="•" <?php selected($toc_h3_symbol, '•'); ?>>•</option>
            <option value="✓" <?php selected($toc_h3_symbol, '✓'); ?>>✓</option>
            <option value="➤" <?php selected($toc_h3_symbol, '➤'); ?>>➤</option>
            <option value="➔" <?php selected($toc_h3_symbol, '➔'); ?>>➔</option>
        </select>
    </label>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
    <label> 글자 크기:
        <input type="range" name="toc_h3_font_size" min="12" max="36" value="<?php echo intval($toc_h3_font_size); ?>" oninput="document.getElementById('h3_font_size_value').innerHTML = this.value + 'px';" />
        <span id="h3_font_size_value"><?php echo esc_attr($toc_h3_font_size); ?></span>
    </label>
    <?php
}

// H4 설정 필드 렌더링
function toc_h4_settings_callback() {
    $toc_h4_symbol = get_option('toc_h4_symbol', '▸');
    $toc_h4_font_size = get_option('toc_h4_font_size', '19px');
    ?>
    <label>기호:
        <select name="toc_h4_symbol">
            <option value="" <?php selected($toc_h4_symbol, ''); ?>>없음</option>
            <option value="▸" <?php selected($toc_h4_symbol, '▸'); ?>>▸</option>
            <option value="•" <?php selected($toc_h4_symbol, '•'); ?>>•</option>
            <option value="✓" <?php selected($toc_h4_symbol, '✓'); ?>>✓</option>
            <option value="➤" <?php selected($toc_h4_symbol, '➤'); ?>>➤</option>
            <option value="➔" <?php selected($toc_h4_symbol, '➔'); ?>>➔</option>
        </select>
    </label>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
    <label> 글자 크기:
        <input type="range" name="toc_h4_font_size" min="12" max="36" value="<?php echo intval($toc_h4_font_size); ?>" oninput="document.getElementById('h4_font_size_value').innerHTML = this.value + 'px';" />
        <span id="h4_font_size_value"><?php echo esc_attr($toc_h4_font_size); ?></span>
    </label>
    <?php
}

// 테이블 배경 색상 필드 렌더링
function toc_table_background_color_callback() {
    $toc_table_background_color = get_option('toc_table_background_color', '#F5F7F8');
    ?>
    <input type="text" name="toc_table_background_color" id="table_bg_color" value="<?php echo esc_attr($toc_table_background_color); ?>" />
    <input type="color" id="table_bg_color_picker" value="<?php echo esc_attr($toc_table_background_color); ?>" />
    <script>
        document.addEventListener('DOMContentLoaded', function () {
            var colorInput = document.getElementById('table_bg_color');
            var colorPicker = document.getElementById('table_bg_color_picker');

            colorPicker.addEventListener('input', function () {
                colorInput.value = colorPicker.value;
            });

            colorInput.addEventListener('input', function () {
                colorPicker.value = colorInput.value;
            });
        });
    </script>
    <?php
}

// 후버 배경 색상 필드 렌더링
function toc_hover_background_color_callback() {
    $toc_hover_background_color = get_option('toc_hover_background_color', '#072A40');
    ?>
    <input type="text" name="toc_hover_background_color" id="hover_bg_color" value="<?php echo esc_attr($toc_hover_background_color); ?>" />
    <input type="color" id="hover_bg_color_picker" value="<?php echo esc_attr($toc_hover_background_color); ?>" />
    <script>
        document.addEventListener('DOMContentLoaded', function () {
            var colorInput = document.getElementById('hover_bg_color');
            var colorPicker = document.getElementById('hover_bg_color_picker');

            colorPicker.addEventListener('input', function () {
                colorInput.value = colorPicker.value;
            });

            colorInput.addEventListener('input', function () {
                colorPicker.value = colorInput.value;
            });
        });
    </script>
    <?php
}

// 목차 생성 함수
function custom_main_toc() {
    ob_start();
    $toc_h2_symbol = get_option('toc_h2_symbol', '');
    $toc_h3_symbol = get_option('toc_h3_symbol', '▸');
    $toc_h4_symbol = get_option('toc_h4_symbol', '▸');
    ?>
    <div id="main-toc">
        <div id="toc2"></div>
    </div>

    <script>
    document.addEventListener('DOMContentLoaded', function () {
        var toc = "<div class='toc-table2'>";
        toc += "<div class='toc-title2'><a href='https://xn--hy1b53d22msjebow.com/' target='_blank'>목차</a></div>";

        var h2Elements = document.querySelectorAll('.entry-content h2');
        var idCount = 0;

        h2Elements.forEach(function (h2, index) {
            if (h2.id === '') {
                h2.id = 'header2-' + idCount++;
            }

            toc += "<div class='toc-column2'><div class='toc2-h2'><a href='#" + h2.id + "'><?php echo esc_js($toc_h2_symbol); ?> " + h2.textContent + "</a></div>";

            var nextElement = h2.nextElementSibling;
            while (nextElement && nextElement.tagName.toLowerCase() !== 'h2') {
                if (nextElement.tagName.toLowerCase() === 'h3' || nextElement.tagName.toLowerCase() === 'h4') {
                    if (nextElement.id === '') {
                        nextElement.id = 'header2-' + idCount++;
                    }
                    if (nextElement.tagName.toLowerCase() === 'h3') {
                        toc += "<div class='toc-box2 toc2-h3'><a href='#" + nextElement.id + "'><?php echo esc_js($toc_h3_symbol); ?> " + nextElement.textContent + "</a></div>";
                    } else if (nextElement.tagName.toLowerCase() === 'h4') {
                        toc += "<div class='toc-box2 toc2-h4'><a href='#" + nextElement.id + "'><?php echo esc_js($toc_h4_symbol); ?> " + nextElement.textContent + "</a></div>";
                    }
                }
                nextElement = nextElement.nextElementSibling;
            }

            toc += "</div>";
        });

        toc += "</div>";

        var tocElement = document.getElementById('toc2');
        if (tocElement) {
            tocElement.innerHTML = toc;
        }

        window.addEventListener('scroll', function() {
            var scrollPosition = window.scrollY || document.documentElement.scrollTop;

            var allHeadings = document.querySelectorAll('.entry-content h2, .entry-content h3, .entry-content h4');

            document.querySelectorAll('#toc2 a').forEach(function(a) {
                a.classList.remove('current-section');
            });

            allHeadings.forEach(function (heading, index) {
                var sectionTop = heading.offsetTop;
                var nextHeading = allHeadings[index + 1];
                var nextSectionTop = nextHeading ? nextHeading.offsetTop : Number.MAX_SAFE_INTEGER;

                if (scrollPosition >= sectionTop && scrollPosition < nextSectionTop) {
                    var currentLink = document.querySelector('#toc2 a[href="#' + heading.id + '"]');
                    if (currentLink) {
                        currentLink.classList.add('current-section');
                    }
                }
            });
        });
    });
    </script>
    <?php
    return ob_get_clean();
}

// 본문에 목차를 자동으로 추가하는 필터 함수
function add_custom_toc_to_content($content) {
    if (is_singular('post')) {
        $toc = custom_main_toc();
        $toc_position = get_option('toc_position', 'after_first_paragraph');

        if ($toc_position == 'top_of_page') {
            $content = $toc . $content;
        } else {
            $paragraphs = explode('</p>', $content);
            if (isset($paragraphs[1])) {
                $paragraphs[1] = $toc . $paragraphs[1];
            }
            $content = implode('</p>', $paragraphs);
        }
    }
    return $content;
}
add_filter('the_content', 'add_custom_toc_to_content');

// 커스텀 목차 스타일
function toc_custom_styles() {
    $toc_h2_font_size = get_option('toc_h2_font_size', '22px');
    $toc_h3_font_size = get_option('toc_h3_font_size', '20px');
    $toc_h4_font_size = get_option('toc_h4_font_size', '19px');
    $toc_table_background_color = get_option('toc_table_background_color', '#F5F7F8');
    $toc_hover_background_color = get_option('toc_hover_background_color', '#072A40');
    $toc_table_width = get_option('toc_table_width', '75%');
    ?>
    <style>
    .toc-table2 {
        width: <?php echo esc_attr($toc_table_width); ?> !important;
        margin: 100px auto 100px auto;
        border: 1px solid #ccc;
        position: relative;
        box-sizing: border-box;
        box-shadow: 0px 4px 4px rgba(0, 0, 0, 0.1);
        background-color: <?php echo esc_attr($toc_table_background_color); ?> !important;
        padding: 15px 5px 0 5px;
    }

    .toc-title2 {
        position: absolute;
        top: -20px;
        left: 50%;
        transform: translateX(-50%);
        background: linear-gradient(#ffffff 50%, <?php echo esc_attr($toc_table_background_color); ?> 50%);
        padding: 0 10px;
        font-weight: bold;
        font-size: 24px !important;
        text-align: center;
        margin-bottom: 50px;
    }

    .toc-title2 a {
        text-decoration: none;
        color: black;
		        font-size: 22px !important;
    }

    .toc-column2 {
        border-bottom: 1px solid #ccc;
        padding-bottom: 15px;
    }

    .toc-box2 {
        margin-left: 4px;
    }

    .toc-box2 a {
        text-decoration: none;
        color: #40444A;
        display: block;
        padding: 5px;
    }

    .toc-box2:hover {
        background-color: <?php echo esc_attr($toc_hover_background_color); ?> !important;
        color: white;
        font-weight: bold;
        border-radius: 4px;
    }

    .toc-box2:hover a {
        color: white !important;
    }

    .toc2-h2 {
        font-size: <?php echo esc_attr($toc_h2_font_size); ?>px !important;
        font-weight: bold;
        margin-top: 10px;
        margin-bottom: 10px;
        text-align: center;
        color: #40444A;
    }

    .toc2-h2 a {
        text-decoration: none;
        color: #40444A !important;
    }

    .toc2-h2:hover {
        background-color: <?php echo esc_attr($toc_hover_background_color); ?> !important;
        color: white;
        border-radius: 4px;
        transition: background-color 0.3s ease;
    }

    .toc2-h2:hover a {
        color: white !important;
    }

    .toc-box2.toc2-h3 {
        font-size: <?php echo esc_attr($toc_h3_font_size); ?>px !important;
        color: #40444A;
        margin: 0;
        margin-left: 5px;
        padding-left: 5px;
    }

    .toc-box2.toc2-h4 {
        font-size: <?php echo esc_attr($toc_h4_font_size); ?>px !important;
        color: #40444A;
        margin: 0;
        margin-left: 20px;
        padding-left: 5px;
    }

    .toc-box2.toc2-h4:hover {
        background-color: <?php echo esc_attr($toc_hover_background_color); ?> !important;
        color: white;
        font-weight: bold;
        border-radius: 4px;
        transition: background-color 0.3s ease;
    }

    .toc-box2.toc2-h4:hover a {
        color: white !important;
    }

    </style>
    <?php
}
add_action('wp_head', 'toc_custom_styles');
?>
